﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
//using System.Threading.Tasks;
using System.Windows.Forms;
using Phidget22;
using Phidget22.Events;
using Phidget22.ExampleUtils;
using System.IO;

namespace Encoder_Example {
	public partial class Form1 : Form {
		CommandLineOpen openArgs;
		Phidget22.Encoder openCh = null;
		Phidget22.Encoder encoder = null;
		formCleaner cleaner;
		private ErrorEventBox errorBox;
		int CPR = -1;
		double prevVelocity = 0;
		double prevTime = 0;

		private DataGraphBox positionGraphBox, velocityGraphBox;

		public Form1() {
			InitializeComponent();
			openArgs = new CommandLineOpen(this);
			cleaner = new formCleaner(this);
		}

		public Form1(Phidget22.Encoder ch) {
			InitializeComponent();
			openCh = ch;
			cleaner = new formCleaner(this);
		}

		private void Form1_Load(object sender, EventArgs e) {
			phidgetInfoBox1.Clear();

			errorBox = new ErrorEventBox(this);
			encoder = openCh ?? openArgs.makeChannel<Phidget22.Encoder>();

			positionGraphBox = new DataGraphBox(this, "Position");
			velocityGraphBox = new DataGraphBox(this, "Velocity");

			encoder.Attach += enc_attach;
			encoder.Detach += enc_detach;
			encoder.Error += enc_error;
			encoder.PositionChange += enc_change;
			encoder.PropertyChange += enc_propertychange;

			try {
				encoder.Open();
			} catch (PhidgetException ex) { errorBox.addMessage("Error opening device: " + ex.Message); }
		}

		private void Form1_FormClosing(object sender, FormClosingEventArgs e) {

			encoder.Attach -= enc_attach;
			encoder.Detach -= enc_detach;
			encoder.Error -= enc_error;
			encoder.PositionChange -= enc_change;
			encoder.PropertyChange -= enc_propertychange;
			encoder.Close();
		}
		void enc_attach(object sender, Phidget22.Events.AttachEventArgs e) {
			try {
				phidgetInfoBox1.FillPhidgetInfo((Phidget)sender);

				Phidget22.Encoder attachedDevice = (Phidget22.Encoder)sender;

				positionGraphBox.SetTitle(DataGraphBox.GenerateTitle("Position", (Phidget)encoder));
				velocityGraphBox.SetTitle(DataGraphBox.GenerateTitle("Velocity", (Phidget)encoder));

				try {
					dataIntervalTrk.Unit = "ms";
					dataIntervalTrk.initializeRange(attachedDevice.MinDataInterval, attachedDevice.MaxDataInterval);
					dataIntervalTrk.Value = attachedDevice.DataInterval;

					//differing max/min means the interval is a settable configuration
					if (attachedDevice.MaxDataInterval == attachedDevice.MinDataInterval)
						dataIntervalTrk.Enabled = false;

					changeTriggerTrk.Unit = " pulses";
					changeTriggerTrk.trkText = changeTriggerTrk.minTxt = attachedDevice.MinPositionChangeTrigger.ToString();
					changeTriggerTrk.Minimum = attachedDevice.MinPositionChangeTrigger;
					changeTriggerTrk.Maximum = attachedDevice.MaxPositionChangeTrigger;
					changeTriggerTrk.maxTxt = attachedDevice.MaxPositionChangeTrigger.ToString();
					changeTriggerTrk.Value = attachedDevice.PositionChangeTrigger;
					changeTriggerTrk.trkText = attachedDevice.PositionChangeTrigger.ToString("F0") + changeTriggerTrk.Unit;

				} catch (PhidgetException ex) {
					errorBox.addMessage("Error setting data interval or change trigger: " + ex.Message);
				}

				try // Populate ioMode control and hide if ioMode is unsupported
				{
					EncoderIOMode io = encoder.IOMode;

					foreach (EncoderIOMode eio in Enum.GetValues(typeof(EncoderIOMode)))
						iomodeCombo.Items.Add(eio);
					iomodeCombo.SelectedIndex = 0;

				} catch (PhidgetException ex) {
					if (ex.ErrorCode == ErrorCode.Unsupported)
						iomodePanel.Hide();
				}

				timeTxt.Clear();
				positionTxt.Clear();
				indexTxt.Clear();
				cumulativePosition.Clear();
				timeTxt.Text = "N/A";
				positionTxt.Text = "N/A";
				indexTxt.Text = "N/A";
				cumulativePosition.Text = "N/A";

				switch (((Phidget)sender).DeviceID) {
				case DeviceID.PN_1047:
				case DeviceID.PN_HIN1101:
					enabledChk.Visible = true;
					enabledChk.Checked = true;
					attachedDevice.Enabled = true;
					break;
				}

				settingsBox.Visible = true;
				outputBox.Visible = true;
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error during attach event: " + ex.Message);
			}
		}
		void enc_detach(object sender, Phidget22.Events.DetachEventArgs e) {
			phidgetInfoBox1.Clear();
			cleaner.clean();

			settingsBox.Visible = false;
			outputBox.Visible = false;

			iomodeCombo.Items.Clear();

			enabledChk.Visible = false;
			cprTxt.Text = "";
			CPR = -1;
			dataIntervalTrk.TrueValue = -1;
		}
		void enc_error(object sender, Phidget22.Events.ErrorEventArgs e) {
			errorBox.addMessage(e.Description);
		}

		private void enc_propertychange(object sender, PropertyChangeEventArgs e)
		{

			switch (e.PropertyName)
			{
				case "DataInterval":
					if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL)
					{
						dataIntervalTrk.Value = encoder.DataInterval;
						updateGraphDataInterval();
					}
					break;
				case "PositionChangeTrigger":
					changeTriggerTrk.Value = encoder.PositionChangeTrigger;
					break;
				case "DataRate":
					if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE)
					{
						dataIntervalTrk.Value = encoder.DataRate;
						updateGraphDataInterval();
					}
					break;
			}
		}
		void enc_change(object sender, Phidget22.Events.EncoderPositionChangeEventArgs e) {
			try {
				positionTxt.Text = e.PositionChange.ToString() + " pulses";
				timeTxt.Text = (e.TimeChange).ToString("F3") + " ms";

				if (e.IndexTriggered) {
					indexTxt.Text = encoder.IndexPosition.ToString() + " pulses";
				}

				cumulativePosition.Text = encoder.Position.ToString() + " pulses";

				if (CPR != -1) {
					if (rotarySelect.Checked)
					{
						double timeChangeMinutes = e.TimeChange / 60000.0; // ms -> minutes
						if (prevVelocity == 0)
							timeChangeMinutes += prevTime;
						prevTime = timeChangeMinutes;
						double velocity = (((double)e.PositionChange / CPR) / timeChangeMinutes);
						rpmTxt.Text = (velocity / 4).ToString("F0") + " RPM"; // /4 to account for cpr vs ppr
					}
					else if (linearSelect.Checked)
					{
						double timeChangeSeconds = e.TimeChange / 1000.0; // ms -> minutes
						if (prevVelocity == 0)
							timeChangeSeconds += prevTime;
						prevTime = timeChangeSeconds;
						double velocity = (((double)e.PositionChange * CPR) / (timeChangeSeconds*1000)); // divide by 1000 to convert micron to mm
						rpmTxt.Text = velocity.ToString("F1") + " mm/s";
					}


					{
						double timeChangeMinutes = e.TimeChange / 60000.0; // ms -> minutes
						if (prevVelocity == 0)
							timeChangeMinutes += prevTime;
						prevTime = timeChangeMinutes;
						double velocity = (((double)e.PositionChange / CPR) / timeChangeMinutes);

						if (Math.Abs(velocity) > 1e5)
							Console.WriteLine(velocity.ToString());
						if (e.PositionChange != 0 || timeChangeMinutes > (1 / 60.0))
							velocityGraphBox.addData(velocity, encoder.DataInterval);
						prevVelocity = velocity;
					}
				}

				positionGraphBox.addData(encoder.Position, encoder.DataInterval);
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error reading position: " + ex.Message);
			}
		}

		private void enabledChk_CheckedChanged(object sender, EventArgs e) {
			try {
				encoder.Enabled = enabledChk.Checked;
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error setting enabled: " + ex.Message);
			}
		}

		private void changeTriggertrk_Scroll(object sender, EventArgs e) {
			if (MouseButtons != MouseButtons.Left) {
				try {
					encoder.PositionChangeTrigger = (int)changeTriggerTrk.Value;
				} catch (Exception ex) {
					errorBox.addMessage("Error setting change trigger: " + ex.Message);
				}
			}
		}

		private void changeTriggertrk_MouseUp(object sender, MouseEventArgs e) {
			try {
				encoder.PositionChangeTrigger = (int)changeTriggerTrk.Value;
			} catch (Exception ex) {
				errorBox.addMessage("Error setting change trigger: " + ex.Message);
			}
		}

		private void cprTxt_KeyDown(object sender, KeyEventArgs e) {
			if (e.KeyCode == Keys.Enter) {
				try {
					if (cprTxt.Text != "" && cprTxt.Text != "0")
						CPR = Int32.Parse(cprTxt.Text);
					else {
						CPR = -1;
						rpmTxt.Text = "N/A";
						cprTxt.Text = "";
					}
				} catch (Exception) {
					errorBox.addMessage("Invalid CPR value.");
					cprTxt.Text = "";
				}
				e.SuppressKeyPress = true;
			}
		}

		private void CPRSetBtn_Click(object sender, EventArgs e) {
			try {
				CPR = Int32.Parse(cprTxt.Text);
			} catch (Exception) {
				errorBox.addMessage("Invalid CPR value.");
				cprTxt.Text = "";
			}
		}

		private void rstBtn_Click(object sender, EventArgs e) {
			try {
				encoder.Position = 0;
				positionGraphBox.addData(0, encoder.DataInterval);
			} catch (Exception ex) {
				errorBox.addMessage("Error setting position: " + ex.Message);
			}
		}
		private void updateGraphDataInterval()
		{
			try
			{
				if (encoder.PositionChangeTrigger == 0)
					positionGraphBox.DataInterval = (1000.0 / encoder.DataRate);
				else
					positionGraphBox.DataInterval = null;
			}
			catch (PhidgetException ex) { errorBox.addMessage("Error updating data interval: " + ex.Message); }
		}
		private void iomodeCombo_SelectedIndexChanged(object sender, EventArgs e) {
			try {
				encoder.IOMode = (EncoderIOMode)iomodeCombo.Items[iomodeCombo.SelectedIndex];
			} catch (Exception ex) {
				errorBox.addMessage("Error setting IO Mode: " + ex.Message);
			}
		}

		private void positionGraphButton_Click(object sender, EventArgs e) {
			if (!positionGraphBox.Visible)
				positionGraphBox.Show();
		}

		private void dataIntervalTrk_KeyOverride(object sender, KeyEventArgs e)
		{
			e.SuppressKeyPress = true;
			dataIntervalTrk.keyOverride(sender, e);
		}

		private void slider_ValueChangeEvent(object sender, EventArgs e)
		{
			if (((PhidgetsControlLibrary.CustomScroll)sender).isFixedRange) //datainterval control
			{
				if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL)
				{
					try
					{
						double temp = dataIntervalTrk.Value;
						encoder.DataInterval = (int)temp;
						if (encoder.DataInterval < (int)temp)
						{
							while (encoder.DataInterval != (int)temp)
							{
								temp++;
								encoder.DataInterval = (int)temp;
							}
						}
						else
						{
							while (encoder.DataInterval != (int)temp)
							{
								temp--;
								encoder.DataInterval = (int)temp;
							}
						}

						dataIntervalTrk.TrueValue = encoder.DataInterval;
						dataIntervalTrk.Value = encoder.DataInterval;



						updateGraphDataInterval();
					}
					catch (Exception ex) { errorBox.addMessage("Error setting data interval: " + ex.Message); }
				}
				else if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE)
				{
					try
					{
						encoder.DataRate = dataIntervalTrk.Value;
						updateGraphDataInterval();
					}
					catch (Exception ex) { errorBox.addMessage("Error setting data rate: " + ex.Message); }
				}
			}
			else //changetrigger control
			{
				try
				{
					encoder.PositionChangeTrigger = (int)changeTriggerTrk.Value;
					updateGraphDataInterval();
				}
				catch (Exception ex) { errorBox.addMessage("Error setting change trigger: " + ex.Message); }
			}
		}
		private void dataIntervalTrk_ModeSwitch(object sender, EventArgs e)
		{
			dataIntervalTrk.SwitchModes();
			if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE)
			{
				try
				{
					dataIntervalTrk.isIntValue = false;
					dataIntervalTrk.Unit = "Hz";
					dataIntervalTrk.trkText = encoder.DataRate.ToString() + "Hz";
					dataIntervalTrk.Value = encoder.DataRate;
				}
				catch (Exception ex) { errorBox.addMessage("Error setting data rate: " + ex.Message); }

			}
			else if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL)
			{
				try
				{
					dataIntervalTrk.isIntValue = true;
					if (encoder.DeviceID != DeviceID.PN_1046)
						dataIntervalTrk.Unit = "ms";
					dataIntervalTrk.trkText = encoder.DataInterval.ToString() + "ms";
					dataIntervalTrk.Value = encoder.DataInterval;
				}
				catch (Exception ex) { errorBox.addMessage("Error setting data interval: " + ex.Message); }
			}
			dataIntervalTrk.TrueValue = dataIntervalTrk.Value;
		}

        private void rotarySelect_CheckedChanged(object sender, EventArgs e)
        {
			if (rotarySelect.Checked)
			{
				if (linearSelect.Checked)
					linearSelect.Checked = false;
			}

			cprLbl.Text = "Cycles per revolution";
			cprTxt.Text = "";
		}

        private void linearSelect_CheckedChanged(object sender, EventArgs e)
        {
			if (linearSelect.Checked)
			{
				if (rotarySelect.Checked)
					rotarySelect.Checked = false;
			}

			cprLbl.Text = "µm per pulse";
			cprTxt.Text = "";
		}

        private void velocityGraphButton_Click(object sender, EventArgs e) {
			if (!velocityGraphBox.Visible)
				velocityGraphBox.Show();
		}
	}
}
